@@ -57,9 +57,9 @@ zend_object *zend_enum_new(zval *result, zend_class_entry *ce, zend_string *case
 	return zobj;
 }
 
-static void zend_verify_enum_properties(zend_class_entry *ce)
+static void zend_verify_enum_properties(const zend_class_entry *ce)
 {
-	zend_property_info *property_info;
+	const zend_property_info *property_info;
 
 	ZEND_HASH_MAP_FOREACH_PTR(&ce->properties_info, property_info) {
 		if (zend_string_equals(property_info->name, ZSTR_KNOWN(ZEND_STR_NAME))) {
@@ -77,7 +77,7 @@ static void zend_verify_enum_properties(zend_class_entry *ce)
 	} ZEND_HASH_FOREACH_END();
 }
 
-static void zend_verify_enum_magic_methods(zend_class_entry *ce)
+static void zend_verify_enum_magic_methods(const zend_class_entry *ce)
 {
 	// Only __get, __call and __invoke are allowed
 
@@ -109,7 +109,7 @@ static void zend_verify_enum_magic_methods(zend_class_entry *ce)
 	}
 }
 
-static void zend_verify_enum_interfaces(zend_class_entry *ce)
+static void zend_verify_enum_interfaces(const zend_class_entry *ce)
 {
 	if (zend_class_implements_interface(ce, zend_ce_serializable)) {
 		zend_error_noreturn(E_COMPILE_ERROR,
@@ -117,7 +117,7 @@ static void zend_verify_enum_interfaces(zend_class_entry *ce)
 	}
 }
 
-void zend_verify_enum(zend_class_entry *ce)
+void zend_verify_enum(const zend_class_entry *ce)
 {
 	zend_verify_enum_properties(ce);
 	zend_verify_enum_magic_methods(ce);
@@ -205,7 +205,7 @@ zend_result zend_enum_build_backed_enum_table(zend_class_entry *ce)
 	zend_hash_init(backed_enum_table, 0, NULL, ZVAL_PTR_DTOR, 0);
 	zend_class_set_backed_enum_table(ce, backed_enum_table);
 
-	zend_string *enum_class_name = ce->name;
+	const zend_string *enum_class_name = ce->name;
 
 	zend_string *name;
 	zval *val;
@@ -228,7 +228,7 @@ zend_result zend_enum_build_backed_enum_table(zend_class_entry *ce)
 
 		if (ce->enum_backing_type == IS_LONG) {
 			zend_long long_key = Z_LVAL_P(case_value);
-			zval *existing_case_name = zend_hash_index_find(backed_enum_table, long_key);
+			const zval *existing_case_name = zend_hash_index_find(backed_enum_table, long_key);
 			if (existing_case_name) {
 				zend_throw_error(NULL, "Duplicate value in enum %s for cases %s and %s",
 					ZSTR_VAL(enum_class_name),
@@ -241,7 +241,7 @@ zend_result zend_enum_build_backed_enum_table(zend_class_entry *ce)
 		} else {
 			ZEND_ASSERT(ce->enum_backing_type == IS_STRING);
 			zend_string *string_key = Z_STR_P(case_value);
-			zval *existing_case_name = zend_hash_find(backed_enum_table, string_key);
+			const zval *existing_case_name = zend_hash_find(backed_enum_table, string_key);
 			if (existing_case_name != NULL) {
 				zend_throw_error(NULL, "Duplicate value in enum %s for cases %s and %s",
 					ZSTR_VAL(enum_class_name),
@@ -294,7 +294,7 @@ ZEND_API zend_result zend_enum_get_case_by_value(zend_object **result, zend_clas
 		}
 	}
 
-	HashTable *backed_enum_table = CE_BACKED_ENUM_TABLE(ce);
+	const HashTable *backed_enum_table = CE_BACKED_ENUM_TABLE(ce);
 	if (!backed_enum_table) {
 		goto not_found;
 	}
@@ -421,7 +421,11 @@ static void zend_enum_register_func(zend_class_entry *ce, zend_known_string_id n
     if (EG(active)) { // at run-time
 		ZEND_MAP_PTR_INIT(zif->run_time_cache, zend_arena_calloc(&CG(arena), 1, zend_internal_run_time_cache_reserved_size()));
 	} else {
-		ZEND_MAP_PTR_NEW(zif->run_time_cache);
+#ifdef ZTS
+		ZEND_MAP_PTR_NEW_STATIC(zif->run_time_cache);
+#else
+		ZEND_MAP_PTR_INIT(zif->run_time_cache, NULL);
+#endif
 	}
 
 	if (!zend_hash_add_ptr(&ce->function_table, name, zif)) {
@@ -437,6 +441,7 @@ void zend_enum_register_funcs(zend_class_entry *ce)
 	cases_function->handler = zend_enum_cases_func;
 	cases_function->function_name = ZSTR_KNOWN(ZEND_STR_CASES);
 	cases_function->fn_flags = fn_flags;
+	cases_function->doc_comment = NULL;
 	cases_function->arg_info = (zend_internal_arg_info *) (arginfo_class_UnitEnum_cases + 1);
 	zend_enum_register_func(ce, ZEND_STR_CASES, cases_function);
 
@@ -445,6 +450,7 @@ void zend_enum_register_funcs(zend_class_entry *ce)
 		from_function->handler = zend_enum_from_func;
 		from_function->function_name = ZSTR_KNOWN(ZEND_STR_FROM);
 		from_function->fn_flags = fn_flags;
+		from_function->doc_comment = NULL;
 		from_function->num_args = 1;
 		from_function->required_num_args = 1;
 		from_function->arg_info = (zend_internal_arg_info *) (arginfo_class_BackedEnum_from + 1);
@@ -454,6 +460,7 @@ void zend_enum_register_funcs(zend_class_entry *ce)
 		try_from_function->handler = zend_enum_try_from_func;
 		try_from_function->function_name = ZSTR_KNOWN(ZEND_STR_TRYFROM);
 		try_from_function->fn_flags = fn_flags;
+		try_from_function->doc_comment = NULL;
 		try_from_function->num_args = 1;
 		try_from_function->required_num_args = 1;
 		try_from_function->arg_info = (zend_internal_arg_info *) (arginfo_class_BackedEnum_tryFrom + 1);
@@ -541,12 +548,14 @@ static zend_ast_ref *create_enum_case_ast(
 	ast->child[0]->attr = 0;
 	ZEND_ASSERT(ZSTR_IS_INTERNED(class_name));
 	ZVAL_STR(zend_ast_get_zval(ast->child[0]), class_name);
+	Z_LINENO_P(zend_ast_get_zval(ast->child[0])) = 0;
 
 	ast->child[1] = (zend_ast *) p; p += sizeof(zend_ast_zval);
 	ast->child[1]->kind = ZEND_AST_ZVAL;
 	ast->child[1]->attr = 0;
 	ZEND_ASSERT(ZSTR_IS_INTERNED(case_name));
 	ZVAL_STR(zend_ast_get_zval(ast->child[1]), case_name);
+	Z_LINENO_P(zend_ast_get_zval(ast->child[1])) = 0;
 
 	if (value) {
 		ast->child[2] = (zend_ast *) p; p += sizeof(zend_ast_zval);
@@ -554,6 +563,7 @@ static zend_ast_ref *create_enum_case_ast(
 		ast->child[2]->attr = 0;
 		ZEND_ASSERT(!Z_REFCOUNTED_P(value));
 		ZVAL_COPY_VALUE(zend_ast_get_zval(ast->child[2]), value);
+		Z_LINENO_P(zend_ast_get_zval(ast->child[2])) = 0;
 	} else {
 		ast->child[2] = NULL;
 	}
